
% -------------------------------------------------------------------------
% EMI Toolbox - STEP1_ExtentComputation.m
% -------------------------------------------------------------------------
% Description
%
% Compute the experimental extents and associated (approximate) confidence
% intervals.
%
% -------------------------------------------------------------------------
% Syntax
%
% I/O:	Initial
%
% -------------------------------------------------------------------------
% Last modification: Kris Villez, 2016-05-11
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2015-2016 Alma Masic, Kris Villez
%
% This file is part of the EMI toolbox for efficient model identification. 
% 
% The EMI Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your
% option) any later version. 
% 
% The EMI Toolbox is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with the EMI Toolbox. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

Initial

A=sprintf([ '\n'...
            'This script computes experimental extents for three sensor \n' ...
            'layouts. \n'            ]);
disp(A)
disp(sprintf('Press any key to continue\n'))
%pause

% -------------------------------------------------------------------------
% Load results and gather data

disp(' ')
disp('=========================')
disp(['   Load data'])
disp('=========================')
disp(' ')

FILEPATH = [ FOLDER 'STEP0_ExperimentalData'] ;
load(FILEPATH)
    

% -------------------------------------------------------------------------
% Setup ODE solver - used for dynamics of component and kinetic species
% concentrations
AbsTol			=	1e-9	;
RelTol			= 	1e-9	;

global optionsode
optionsode = odeset('RelTol',RelTol,'AbsTol',AbsTol); 	

% =========================================================================
% =========================================================================
% =========================================================================

for SensorSetup=3
    
    switch SensorSetup
        case 1 % Case 1: TAN and TNO3
            
            IncMeas	=	[   1 3	]   ;
            pHmeas  =       0       ;
            
        case 2 % Case 2: TAN, TNO2, and TNO3
            
            IncMeas	=	[	1:3	]   ;
            pHmeas	=       0       ;
            
        case 3 % Case 3: TAN, TNO2, TNO3, and pH
            
            IncMeas	=	[	1:4	]   ;
            pHmeas	=       4       ;
            
        case 4 % Case 4: TAN, TNO3, and pH
            
            IncMeas	=	[	2:4	]   ;
            pHmeas	=       4       ;
            
        case 5 % Case 5: TNO3 and pH
            
            IncMeas	=	[	3:4	]   ;
            pHmeas	=       4       ;
    end
    
    %======================================================================
    %======================================================================
    %======================================================================
    
    % ---------------------------------------------------------------------
    % Setup and preparation
    
    disp(' ')
    disp('=========================')
    disp(['   Measurement Setup: ' num2str(SensorSetup) '  '])
    disp('=========================')
    disp(' ')
    
    Rk	=   size(Model.Nbar_k,1)        ;
    H	=	size(Model.data.ytilde,2)	;
    MG  =   Model.dimensions.MG         ;
    
    Model1                      =	Model                               ; 
    Model1.data.ytilde          =	Model.data.ytilde(IncMeas,:)        ;
    Model1.Measurement.sigma	=	Model.Measurement.sigma(IncMeas)	;
    
    Inc                         =	ismember(   IncMeas,   1:MG)        ;
    Select                      =	IncMeas(Inc)                        ;
    Model1.Measurement.G        =	Model.Measurement.G(Select,:)       ;
    
    Model1.dimensions.MG        =   length(Select)                      ;
    Model1.dimensions.M     	=   length(Select)+1                    ;
    
    
    % -------------------------------------------------------------------------
    % Setup candidate rate laws
    
    disp(' ')
    disp('==================================')
    disp(['   Setting up Rate Law Library'])
    disp('==================================')
    disp(' ')
    
    J       =   5           ;
    Library	=	cell(J,1)	;
    
    for i=1:Rk
        
        for j =1:J
            
            Range = [1e-6   100] ;
            switch j
                case 1
                    name       =   'Zero-Order'       ;   % Name
                    ratefun	=	@(S,theta)  (S>0)/theta(1) ;
                    ntheta = 1 ;
                    if i==2
                        Range = [2.5  100] ; % This results in a very stiff system for reaction 2. Parameter range adjusted in view of simulation time.
                    end
                case 2
                    name       =   'First-Order'       ;   % Name
                    ratefun	=	@(S,theta) sign(S).*abs(S)/theta(1) ;
                    ntheta = 1 ;
                case 3
                    name       =   'Monod'       ;   % Name
                    ratefun	=	@(S,theta) sign(S).*abs(S)./((theta(1))+(theta(2))*abs(S)) ;
                    ntheta = 2 ;
                case 4
                    name       =   'Tessier'       ;   % Name
                    ratefun	=	@(S,theta) sign(S).*1/theta(2)*(1-exp(-abs(S)*theta(2)/theta(1)));
                    ntheta = 2 ;
                case 5
                    name       =   'Haldane'       ;   % Name
                    ratefun	=	@(S,theta) sign(S)*(abs(S)./((theta(1))+(theta(2))*abs(S)+(theta(3))*abs(S).^2)) ;
                    ntheta = 3 ;
            end
            
            Library{j,i}.name = name ;
            Library{j,i}.ratefun = ratefun ;
            Library{j,i}.ntheta = ntheta ;
            Library{j,i}.Omega = Range(:)*ones(1,ntheta)            ;
            
        end
    end
    
    
    
    optionsfminsearch               =	optimset('fminsearch')	;
    optionsfminsearch.Display       =	'iter'                  ;
    optionsfminsearch.MaxFunEvals	=	 inf                     ;
    optionsfminsearch.MaxIter       =	 inf                     ;
    optionsfminsearch.MaxIter       =	 11                     ;
    
    %======================================================================
    %======================================================================
    %======================================================================
    
    % ---------------------------------------------------------------------
    % Select model structure
    nModel = J^Rk ;
    
    for iModel=15 ; %1:nModel
        
        [   iModel	nModel	]
        
        num         =	iModel-1	;

        nTheta          =   0                       ;
        Theta0          =   nan(73,1)               ;

        for i=1:Rk
            j       =   mod(num,J)	;
            num     =	num-j       ;
            num     =	num/J       ;
            
            ntheta	=	Library{j+1,i}.ntheta	;
            Theta2r	=	nTheta+(1:ntheta)       ;
            nTheta	=	nTheta+ntheta           ;
            
            theta	=       Model.Reactions{i}.theta        ;
            %theta	=       Library{j,i}.Omega(1,:)         ;
            theta	=	[	theta	zeros(1,10)         ]   ;
            
            Theta0(Theta2r,1)           =	theta(1:ntheta)         ;
            
            Model1.Reactions{i}.name	=	Library{j+1,i}.name     ;
            Model1.Reactions{i}.ratefun =	Library{j+1,i}.ratefun	;
            Model1.Reactions{i}.Theta2r	=	Theta2r                 ;
            
        end
        
        % -----------------------------------------------------------------
        % Actual optimization
        
        WSSRfun                         =	@(t) LackOfFit(Model1,t)                 ;
        Theta0                          =	Theta0(1:nTheta)	;
        [Theta1,fval,exitflag,output]	=	fminsearch(@(t) WSSRfun(t),Theta0,optionsfminsearch)	;
        
        % -----------------------------------------------------------------
        % Store results
        disp(['---  Saving Results'])
        
        FILEPATH	=   [	FOLDER 'Conventional_STEP1_Model' num2str(iModel) ] ;
        save(FILEPATH,'Model1','Theta1')
        
        disp([' '])
        
    end
    
    return
    
    
    
%     % ---------------------------------------------------------------------
%     % Computation
%     
%     disp(['---  Computing Extents'])
%     
%     [xtilde,Lambda]     =	Measurement_2_Extent(Model1,IncMeas,pHmeas)	;
%     
%     lambda              =	nan(Rk,H)                                   ;
%     for iR=1:Rk
%         lambda(iR,:)	=	squeeze(Lambda(iR,iR,:)).^(1/2)             ;
%     end
%      
%     % ---------------------------------------------------------------------
%     % Store results
%     
%     disp(['---  Saving Results'])
%     
%     FILEPATH	=   [	FOLDER 'STEP1_ExtentComputation_Case' num2str(SensorSetup) ] ;   
%     save(FILEPATH,'Model','xtilde','Lambda','lambda')
%     
%     disp([' '])
    
end


% =========================================================================
% =========================================================================
% =========================================================================

STEP1_ExtentComputation_Display

% =========================================================================
% =========================================================================
% =========================================================================
