
% -------------------------------------------------------------------------
% EMI Toolbox - STEP1_ExtentComputation.m
% -------------------------------------------------------------------------
% Description
%
% Compute the experimental extents and associated (approximate) confidence
% intervals.
%
% -------------------------------------------------------------------------
% Syntax
%
% I/O:	Initial
%
% -------------------------------------------------------------------------
% Last modification: Kris Villez, 2016-05-11
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2015-2016 Alma Masic, Kris Villez
%
% This file is part of the EMI toolbox for efficient model identification. 
% 
% The EMI Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your
% option) any later version. 
% 
% The EMI Toolbox is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with the EMI Toolbox. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

Initial

A=sprintf([ '\n'...
            'This script computes experimental extents for three sensor \n' ...
            'layouts. \n'            ]);
disp(A)
disp(sprintf('Press any key to continue\n'))
%pause

% -------------------------------------------------------------------------
% Load results and gather data

disp(' ')
disp('=========================')
disp(['   Load data'])
disp('=========================')
disp(' ')

FILEPATH = [ FOLDER 'STEP0_ExperimentalData'] ;
load(FILEPATH)
    

% -------------------------------------------------------------------------
% Setup ODE solver - used for dynamics of component and kinetic species
% concentrations
AbsTol			=	1e-9	;
RelTol			= 	1e-9	;

global optionsode
optionsode = odeset('RelTol',RelTol,'AbsTol',AbsTol); 	
GrayScale   =	true	;



FS      =   17  ;
LW      =   3	;
LW2     =   2	;
LW = [ 1 1 1 ];
MS = [ 11 7 7];
gN4mol	=	14	;
factor	=	[ gN4mol*ones(1,3) 1/2 ]    ;
Ylim	=	[-.365 5.865]               ;

if GrayScale
    Colours	=   {.73*ones(1,3),.55*ones(1,3),.37*ones(1,3),'k'} ; % For paper (gray-scale)
else
    Colours	=   {[0.635  0.078 0.184 ],[0.929 0.694 0.125],[0 0.447 0.741],'k'} ; % For demos
end

Strings =   {'^','s','v','o'}   ;
Lines	=	{'-','--',':','.'}   ;
% =========================================================================
% =========================================================================
% =========================================================================

for SensorSetup=3
    
    switch SensorSetup
        case 1 % Case 1: TAN and TNO3
            
            IncMeas	=	[   1 3	]   ;
            pHmeas  =       0       ;
            
        case 2 % Case 2: TAN, TNO2, and TNO3
            
            IncMeas	=	[	1:3	]   ;
            pHmeas	=       0       ;
            
        case 3 % Case 3: TAN, TNO2, TNO3, and pH
            
            IncMeas	=	[	1:4	]   ;
            pHmeas	=       4       ;
            
        case 4 % Case 4: TAN, TNO3, and pH
            
            IncMeas	=	[	2:4	]   ;
            pHmeas	=       4       ;
            
        case 5 % Case 5: TNO3 and pH
            
            IncMeas	=	[	3:4	]   ;
            pHmeas	=       4       ;
    end
    
    %======================================================================
    %======================================================================
    %======================================================================
    
    % ---------------------------------------------------------------------
    % Setup and preparation
    
    disp(' ')
    disp('=========================')
    disp(['   Measurement Setup: ' num2str(SensorSetup) '  '])
    disp('=========================')
    disp(' ')
    
    Rk      =   size(Model.Nbar_k,1)        ;
    [M,H]	=	size(Model.data.ytilde)     ;
    MG      =   Model.dimensions.MG         ;
    
    Model1                      =	Model                               ; 
    Model1.data.ytilde          =	Model.data.ytilde(IncMeas,:)        ;
    Model1.Measurement.sigma	=	Model.Measurement.sigma(IncMeas)	;
    
    Inc                         =	ismember(   IncMeas,   1:MG)        ;
    Select                      =	IncMeas(Inc)                        ;
    Model1.Measurement.G        =	Model.Measurement.G(Select,:)       ;
    
    Model1.dimensions.MG        =   length(Select)                      ;
    Model1.dimensions.M     	=   length(Select)+1                    ;
    
    
    % -------------------------------------------------------------------------
    % Setup candidate rate laws
    
    disp(' ')
    disp('==================================')
    disp(['   Setting up Rate Law Library'])
    disp('==================================')
    disp(' ')
    
    J       =   5           ;
    Library	=	cell(J,1)	;
    
    for i=1:Rk
        
        for j =1:J
            
            Range = [1e-6   100] ;
            switch j
                case 1
                    name       =   'Zero-Order'       ;   % Name
                    ratefun	=	@(S,theta)  (S>0)/theta(1) ;
                    ntheta = 1 ;
                    if i==2
                        Range = [2.5  100] ; % This results in a very stiff system for reaction 2. Parameter range adjusted in view of simulation time.
                    end
                case 2
                    name       =   'First-Order'       ;   % Name
                    ratefun	=	@(S,theta) sign(S).*abs(S)/theta(1) ;
                    ntheta = 1 ;
                case 3
                    name       =   'Monod'       ;   % Name
                    ratefun	=	@(S,theta) sign(S).*abs(S)./((theta(1))+(theta(2))*abs(S)) ;
                    ntheta = 2 ;
                case 4
                    name       =   'Tessier'       ;   % Name
                    ratefun	=	@(S,theta) sign(S).*1/theta(2)*(1-exp(-abs(S)*theta(2)/theta(1)));
                    ntheta = 2 ;
                case 5
                    name       =   'Haldane'       ;   % Name
                    ratefun	=	@(S,theta) sign(S)*(abs(S)./((theta(1))+(theta(2))*abs(S)+(theta(3))*abs(S).^2)) ;
                    ntheta = 3 ;
            end
            
            Library{j,i}.name = name ;
            Library{j,i}.ratefun = ratefun ;
            Library{j,i}.ntheta = ntheta ;
            Library{j,i}.Omega = Range(:)*ones(1,ntheta)            ;
            
        end
    end
    
    
    
    optionsfminsearch               =	optimset('fminsearch')	;
    optionsfminsearch.Display       =	'iter'                  ;
    optionsfminsearch.MaxFunEvals	=	 inf                     ;
    optionsfminsearch.MaxIter       =	 inf                     ;
    optionsfminsearch.MaxIter       =	 11                     ;
    
    %======================================================================
    %======================================================================
    %======================================================================
    
    % ---------------------------------------------------------------------
    % Select model structure
    nModel = J^Rk ;
    
    WSSRs = nan(nModel,1) ;
    
    for iModel=1:nModel
        
        [   iModel	nModel	]
        
        % -----------------------------------------------------------------
        % Load results
        disp(['---  Loading Results'])
        
        FILEPATH	=   [	FOLDER 'Conventional_STEP1_Model' num2str(iModel) ] ;
        load(FILEPATH)
        
        ytilde1 =   Model1.data.ytilde                                      ;
        ysim	=	Equation_Measurement(Model1,Model1.simulation.csim)     ;
        
        tsim = 24*Model1.Measurement.tsim ;
        
        index = Model.Measurement.sample_index ;
         tmeas = tsim(index);
        
        fighan = figure;
        set(gcf,'Position',[100 100 1000 800 ])
        
        grid on
        hold on
        Ylim	=	[-.365 5.865]               ;
        set(gca,'FontSize',FS,'Xlim',[0 10],'Xtick',[0:10],'Ylim',Ylim,'Ytick',[0:.5:5.5])
        
        iy=1 ;
        for iy=1:4
            plot(-1-tmeas,ytilde1(iy,:)*factor(iy),'o','Color',Colours{iy},'LineWidth',LW(1),'MarkerSize',MS(2),'MarkerFaceColor','w')
            plot(-1-tsim,ysim(iy,:)*factor(iy),'+-','Color',Colours{iy},'LineWidth',LW(1),'MarkerSize',MS(3))
        end
        
        for iy=1:4
            plot(tsim,ysim(iy,:)*factor(iy),'-','Color',Colours{iy},'LineWidth',LW(1),'MarkerSize',MS(3))
        end
        for iy=1:4
            plot(tmeas,ytilde1(iy,:)*factor(iy),'o','Color',Colours{iy},'LineWidth',LW(1),'MarkerSize',MS(2),'MarkerFaceColor','w')
            plot(tmeas,ysim(iy,index)*factor(iy),'+','Color',Colours{iy},'LineWidth',LW(1),'MarkerSize',MS(3))
        end
        
        xlabel({'Time [h]'})
        ylabel({'Nitrogen concentration [g N/L]'},'Position',[-1 mean(Ylim)])
        legend({...
            '$TAN$ - measurement', '$TAN$ - model',...
            '$TNO2$ - measurement', '$TNO2$ - model',...
            '$TNO3$ - measurement', '$TNO3$ - model',...
            '$pH$ - measurement', '$pH$ - model',...
            },'fontsize',FS,'Color','w','Interpreter','Latex')
        title({ ['Rate law 1: ' Model1.Reactions{1}.name  ],...
                ['Rate law 2: ' Model1.Reactions{2}.name  ]})
        
        set(gca,'Position',[ .1 .1  .8 .8])
        
        axes('Color','none')
        
        Ylim = [-.73 11.73];
        
        set(gca,'FontSize',FS,'Xlim',[0 10],'Xtick',[0:10],'Xticklabel',[ ])
        ylabel({'pH [-]'},'Position',[10.5 5.5],'VerticalAlignment','middle','HorizontalAlignment','center')
        set(gca,'Ylim',Ylim,'Ytick',[0:1:11],'Yaxislocation','right')
        set(gca,'Position',[ .1 .1  .8 .8])
        
        drawnow
        
        SaveMyFigure(fighan,['.\Figures\Results_Method1_Model'  num2str(iModel) ])
        
        
    end
    
    
end


% =========================================================================
% =========================================================================
% =========================================================================
