
% -------------------------------------------------------------------------
% EMI Toolbox - STEP1_ExtentComputation.m
% -------------------------------------------------------------------------
% Description
%
% Compute the experimental extents and associated (approximate) confidence
% intervals.
%
% -------------------------------------------------------------------------
% Syntax
%
% I/O:	Initial
%
% -------------------------------------------------------------------------
% Last modification: Kris Villez, 2016-05-11
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2015-2016 Alma Masic, Kris Villez
%
% This file is part of the EMI toolbox for efficient model identification. 
% 
% The EMI Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your
% option) any later version. 
% 
% The EMI Toolbox is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with the EMI Toolbox. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

Initial

A=sprintf([ '\n'...
            'This script computes experimental extents for three sensor \n' ...
            'layouts. \n'            ]);
disp(A)
disp(sprintf('Press any key to continue\n'))
%pause

% -------------------------------------------------------------------------
% Load results and gather data

disp(' ')
disp('=========================')
disp(['   Load data'])
disp('=========================')
disp(' ')

FILEPATH = [ FOLDER 'STEP0_ExperimentalData'] ;
load(FILEPATH)
    

% -------------------------------------------------------------------------
% Setup ODE solver - used for dynamics of component and kinetic species
% concentrations
AbsTol			=	1e-9	;
RelTol			= 	1e-9	;

global optionsode
optionsode = odeset('RelTol',RelTol,'AbsTol',AbsTol); 	

% =========================================================================
% =========================================================================
% =========================================================================

for SensorSetup=3
    
    switch SensorSetup
        case 1 % Case 1: TAN and TNO3
            
            IncMeas	=	[   1 3	]   ;
            pHmeas  =       0       ;
            
        case 2 % Case 2: TAN, TNO2, and TNO3
            
            IncMeas	=	[	1:3	]   ;
            pHmeas	=       0       ;
            
        case 3 % Case 3: TAN, TNO2, TNO3, and pH
            
            IncMeas	=	[	1:4	]   ;
            pHmeas	=       4       ;
            
        case 4 % Case 4: TAN, TNO3, and pH
            
            IncMeas	=	[	2:4	]   ;
            pHmeas	=       4       ;
            
        case 5 % Case 5: TNO3 and pH
            
            IncMeas	=	[	3:4	]   ;
            pHmeas	=       4       ;
    end
    
    %======================================================================
    %======================================================================
    %======================================================================
    
    % ---------------------------------------------------------------------
    % Setup and preparation
    
    disp(' ')
    disp('=========================')
    disp(['   Measurement Setup: ' num2str(SensorSetup) '  '])
    disp('=========================')
    disp(' ')
    
    Rk      =   size(Model.Nbar_k,1)        ;
    [M,H]	=	size(Model.data.ytilde)     ;
    MG      =   Model.dimensions.MG         ;
    
    Model1                      =	Model                               ; 
    Model1.data.ytilde          =	Model.data.ytilde(IncMeas,:)        ;
    Model1.Measurement.sigma	=	Model.Measurement.sigma(IncMeas)	;
    
    Inc                         =	ismember(   IncMeas,   1:MG)        ;
    Select                      =	IncMeas(Inc)                        ;
    Model1.Measurement.G        =	Model.Measurement.G(Select,:)       ;
    
    Model1.dimensions.MG        =   length(Select)                      ;
    Model1.dimensions.M     	=   length(Select)+1                    ;
    
    
    % -------------------------------------------------------------------------
    % Setup candidate rate laws
    
    disp(' ')
    disp('==================================')
    disp(['   Setting up Rate Law Library'])
    disp('==================================')
    disp(' ')
    
    J       =   5           ;
    Library	=	cell(J,1)	;
    
    for i=1:Rk
        
        for j =1:J
            
            Range = [1e-6   100] ;
            switch j
                case 1
                    name       =   'Zeroth-Order'       ;   % Name
                    ratefun	=	@(S,theta)  (S>0)/theta(1) ;
                    ntheta = 1 ;
                    if i==2
                        Range = [2.5  100] ; % This results in a very stiff system for reaction 2. Parameter range adjusted in view of simulation time.
                    end
                case 2
                    name       =   'First-Order'       ;   % Name
                    ratefun	=	@(S,theta) sign(S).*abs(S)/theta(1) ;
                    ntheta = 1 ;
                case 3
                    name       =   'Monod'       ;   % Name
                    ratefun	=	@(S,theta) sign(S).*abs(S)./((theta(1))+(theta(2))*abs(S)) ;
                    ntheta = 2 ;
                case 4
                    name       =   'Tessier'       ;   % Name
                    ratefun	=	@(S,theta) sign(S).*1/theta(2)*(1-exp(-abs(S)*theta(2)/theta(1)));
                    ntheta = 2 ;
                case 5
                    name       =   'Haldane'       ;   % Name
                    ratefun	=	@(S,theta) sign(S)*(abs(S)./((theta(1))+(theta(2))*abs(S)+(theta(3))*abs(S).^2)) ;
                    ntheta = 3 ;
            end
            
            Library{j,i}.name = name ;
            Library{j,i}.ratefun = ratefun ;
            Library{j,i}.ntheta = ntheta ;
            Library{j,i}.Omega = Range(:)*ones(1,ntheta)            ;
            
        end
    end
    
    
    
    optionsfminsearch               =	optimset('fminsearch')	;
    optionsfminsearch.Display       =	'iter'                  ;
    optionsfminsearch.MaxFunEvals	=	 inf                     ;
    optionsfminsearch.MaxIter       =	 inf                     ;
    optionsfminsearch.MaxIter       =	 11                     ;
    
    %======================================================================
    %======================================================================
    %======================================================================
    
    % ---------------------------------------------------------------------
    % Select model structure
    nModel = J^Rk ;
    
    WSSRs = nan(nModel,1) ;
    
    for iModel=1:nModel
        
        [   iModel	nModel	]
        
        % -----------------------------------------------------------------
        % Load results
        disp(['---  Loading Results'])
        
        FILEPATH	=   [	FOLDER 'Conventional_STEP1_Model' num2str(iModel) ] ;
        load(FILEPATH)
        
        WSSRs(iModel) = WSSR ;
        
    end
    
    WSSRs = reshape(WSSRs,[5 5]) ;
    WSSRs = WSSRs' ;
    WSSRs = WSSRs(:) ;
    
    fr      =   [0:4]'/4    ;
    colors	=   fr*[1 1 1] ;
    strings = {'+','x','^','v','*'} ;
    
    
    figure, hold on
    set(gcf,'Position',[100 100 1000 800 ])
    set(gca,'FontSize',18)

    for iModel=1:nModel
        
        iCol = mod(iModel-1,5) ;
        hb = bar(iModel, sqrt(WSSRs(iModel)/(M*H)), 'facecolor', colors(iCol+1,:));
        
    end
    
    for iModel=1:nModel
        
        plot(iModel,3,'wo','MarkerFaceColor','w','MarkerSize',12)
        plot(iModel,3,['k' strings{ceil(iModel/5)}],'MarkerSize',8)
        
    end
    
    addpath('C:\Z\Research\Code\Tools\backpack\columnlegend\columnlegend')
    
    leg_names = {'Rate law 2: Zeroth-order','Rate law 2: First-order','Rate law 2: Monod','Rate law 2: Tessier','Rate law 2: Haldane'} ;
    legend(leg_names,'Location','North')
    set(gca,'Xlim',[0 26],'Xtick',[1:25],'Xticklabel',[],'TickDir','out')
    set(gca,'Ylim',[0 35])
    plot([0.5 25.5],[ 1 1 ]*sqrt(250.2554/(M*H)),'-','LineWidth',8,'Color',[1 1 1]*.73)
    plot([0.5 25.5],[ 1 1 ]*sqrt(244.6243/(M*H)),':','LineWidth',3,'Color',[1 1 1]*.23)
    xlabel('Model')
    ylabel('WRMSR')
    
    pause
    
    hax2 = axes('Color','none')  ;
    hold on
    set(gca,'FontSize',18)
    
    set(gca,'Xlim',[0 26],'Xtick',[1:25],'Xticklabel',[],'TickDir','out') 
    set(gca,'Ylim',[0 35],'Ytick',[])
    
    for j=1:5
        
        plot(j,-1,['k' strings{j}],'MarkerSize',8)
        
    end
    
    leg_names = {'Rate law 1: Zeroth-order','Rate law 1: First-order','Rate law 1: Monod','Rate law 1: Tessier','Rate law 1: Haldane'} ;
    hleg2=legend(leg_names,'Location','NorthWest');
    set(hleg2,'Color','w')
     
    hax3 = axes('Color','none')  ;
    hold on
    set(gca,'FontSize',18)
    
    set(gca,'Xlim',[0 26],'Xtick',[1:25],'Xticklabel',[],'TickDir','out')
    set(gca,'Ylim',[0 35],'Ytick',[])
    
    plot([0.5 25.5],[ 1 1 ]*sqrt(250.2554/(M*H)),'-','LineWidth',6,'Color',[1 1 1]*.73)
    plot([0.5 25.5],[ 1 1 ]*sqrt(244.6243/(M*H)),':','LineWidth',3,'Color',[1 1 1]*.23)
    leg_names = {'WRMSR_{true}','WRMSR_{method 2}'} ;
    hleg3=legend(leg_names,'Location','NorthWest');
    set(hleg3,'Color','w')
    
end


% =========================================================================
% =========================================================================
% =========================================================================
