function model = Partitioning (model,options) 

% -------------------------------------------------------------------------
% EMI Toolbox - Partitioning.m
% -------------------------------------------------------------------------
% Description
%
%
%
% -------------------------------------------------------------------------
% Syntax
%
% I/O:	model = Partitioning (model,options) 
%
% INPUT
%   model	:   
%   options	: 
%
% OUTPUT
%	model	:   
%
% -------------------------------------------------------------------------
% Last modification: 2018-12-12
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2018 Kris Villez
%
% This file is part of the EMI toolbox for efficient model identification. 
% 
% The EMI Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your
% option) any later version. 
% 
% The EMI Toolbox is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with the EMI Toolbox. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------


% ------------------------------------------------------------------------
%	Get information from structure
% ------------------------------------------------------------------------

T               =	model.kinetic.T               ;
ratecell        =	model.kinetic.ratecell        ;
name_parameters	=	model.kinetic.name_parameters	;

index_bar_o     =	model.static.index_bar_o      ;
NNbar_o          =	model.static.NNbar_o          ;
NNbar_u          =	model.static.NNbar_u          ;
R               =	model.static.R                ;
R_o             =	model.static.R_o              ;
rho_o           =	model.static.rho_o            ;
vertexnames     =	model.static.varnames         ;
Vo              =	model.static.V_o              ;

verbose         =	options.verbose         ;

% --------------------------------------------------------------------
%   Symbolic analysis to generate graph
% --------------------------------------------------------------------

if options.verbose, disp('PARTITIONING - GRAPH'), end

ratefun         =	@(c,theta) cellfun(@(exp) exp(c,theta), ratecell)   ;

% symbolic variables
%sym_c           =	sym('sym_c',[S 1 ])                 ;
sym_x           =	sym('sym_x',[R 1 ])                 ;
sym_chi_bar_o	=	sym('sym_chi_bar_o',[R_o+rho_o 1 ]) ;
sym_theta       =	sym('sym_theta',[T 1 ])           ;

% graph in matrix form
%   1. initialize
F               =	zeros(R+rho_o+T,R+rho_o+T) ;
%   2. links from extents to extents
for r=1:R
    ds          =   diff(	ratefun(...
                                NNbar_u'*sym_x+NNbar_o'*sym_chi_bar_o,...
                                sym_theta ),...
                            sym_x(r), 1)    ;
    F(r,1:R)	=	(~logical(ds==0)) ;
end
%   3. links from extents to extent directions
F(1:R,R+(1:rho_o)) = (Vo~=0) ;
%   4. links from extent directions to extents
for r=1:(R_o+rho_o)
    ds          =	diff(   ratefun(...
                                NNbar_u'*sym_x+NNbar_o'*sym_chi_bar_o,...
                                sym_theta ),...
                            sym_chi_bar_o(r), 1)    ;
    F(index_bar_o(r),1:R)	=	or(F(index_bar_o(r),1:R),(~logical(ds==0))') ;
end
%   5. links from parameters to extents
for t=1:T
    ds          =   diff(   ratefun(...
                                NNbar_u'*sym_x+NNbar_o'*sym_chi_bar_o,...
                                sym_theta ),...
                            sym_theta(t), 1)    ;
    F(R+rho_o+t,1:R) = (~logical(ds==0)) ;
end

% create directed graph
G       =   digraph(F-diag(diag(F)))      ;

% --------------------------------------------------------------------
%   Graph clustering
% --------------------------------------------------------------------

if options.verbose, disp('PARTITIONING - CLUSTERING'), end

% remove edges from extent directions
F_                  =   F           ;
F_(index_bar_o,:)	=   0           ;

% create directed graph
G_                  =   digraph(F_) ;

% partitioning
partition           =	nan(R+rho_o+T,1)	;
part                =   0               ;
for r=index_bar_o(:)'
    tree            =	shortestpathtree(G_,1:(R+rho_o+T), r)     ;
    vertex          =	unique([ r ; tree.Edges.EndNodes(:,1)]) ;
    labeled         =	~isnan(partition(vertex))               ;
    if any(labeled)
        partition(vertex)	=	min(partition(vertex(labeled)))	;
    else
        part                =   part+1  ;
        partition(vertex)	=   part	;
        
    end
end

% --------------------------------------------------------------------
%   Post-processing
% --------------------------------------------------------------------

j_               =	unique(partition )  ;
j_               =	j_(~isnan(j_))      ;
J               =	length(j_)          ;

% cell array of vertices, each for one type of vertex:
vrtx_interp         =	cell(J,1)       ;   % interpolated extent vertices
vrtx_sim_ext        =	cell(J,1)       ;   % simulated extent vertices
vrtx_sim_dir        =	cell(J,1)       ;   % simulated extent direction vertices
vrtx_par            =	cell(J,1)       ;   % kinetic parameter vertices

% add vertices for every subsystem:
for j=1:J
    
    vrtx_target     =	find(partition==j)                      ;
    Fp              =	F(:,vrtx_target)                        ;
    vrtx_source     =	find(any(Fp,2))                         ;
    
    vrtx_interp{j}	=	setdiff(vrtx_source,vrtx_target)        ;
    vrtx_par{j}     =	intersect(vrtx_target,R+rho_o+(1:T))    ;
    vrtx_sim_ext{j}	=	intersect(vrtx_target,1:R)              ;
    vrtx_sim_dir{j}	=	intersect(vrtx_target,R+(1:rho_o))      ;
    
end

% select names for the vertices:
for t=1:T
    vertexnames{R+rho_o+t} =  name_parameters{t};
end

% ------------------------------------------------------------------------
%	Output
% ------------------------------------------------------------------------

% Save results:
model.kinetic.graph.G               =	G               ;
model.kinetic.graph.vertexnames     =	vertexnames     ;
model.kinetic.graph.vrtx_interp     =	vrtx_interp     ;
model.kinetic.graph.vrtx_sim_ext	=	vrtx_sim_ext	;
model.kinetic.graph.vrtx_sim_dir	=	vrtx_sim_dir	;
model.kinetic.graph.vrtx_par        =	vrtx_par        ;
model.kinetic.partition             =	partition       ;

% Command line output:
if verbose>=2
    substr = {'\','{','}'};
    disp(['	Number of subsystems: ' num2str(J)])
    
    for j=j_'
        disp(['	Subsystem: ' num2str(j)])
        disp(['     Simulated extents:     ' StripString(sprintf('%s ',vertexnames{vrtx_sim_ext{j}}),substr)	]   )
        disp(['     Simulated directions:  ' StripString(sprintf('%s ',vertexnames{vrtx_sim_dir{j}}),substr)	]   )
        disp(['     Interpolation:         ' StripString(sprintf('%s ',vertexnames{vrtx_interp{j}}),substr)	]   )
        disp(['     Parameters:            ' StripString(sprintf('%s ',vertexnames{vrtx_par{j}}),substr)      ]   )
    end
end

